.TH BACKDRAW 1 "Release 1.1" "X11 R5; OSF/Motif 1.2"
.ds )H Eindhoven University of Technology
.ds ]W Aug 1992
.SH NAME
backdraw - Backend Drawing Utility
.SH SYNOPSIS
.ta 8n
\fBbackdraw\fP  [-\fI<toolkit option>\fP ...]  [-\fI<backdraw option>\fP ...] 
.SH DESCRIPTION
.I Backdraw
is a utility that reads commands from standard input to show a picture on
the screen. It accepts a number of predefined commands that show or
manipulate a picture on the screen.
.PP
Two different policies are used to show a picture with different colors
and fonts on the screen. This can be specified using the \fIplaneGC\fP
resource.
If \fBplaneGC\fP is set to True, a unique graphics context
with a default color and font will be created for every plane. The user has to
set the default color of every plane only once. If the default plane is
changed, the appropriate graphics context will be used to show new graphics
objects on the screen. It is allowed to use multiple colors per plane.
Changing the color will change the default color used for that plane.
.br
If \fBplaneGC\fP is set to False,
a single graphics context will be used to show
graphics objects on the screen. As a result the user has to change both
color and plane, if he wants to add an object with a specific color to
a plane.
.PP
.I Backdraw
is written in ANSI C and uses the OSF/Motif Release 1.2
toolkit and the X Windows Version 11 Release 5 software.
It is also compatible with older releases of X Windows and the
OSF/Motif toolkit.
.I Backdraw
runs on various hardware platforms. It has been compiled and
tested on Apollo Domain OS 10.4 and HP Series 700 machines.
It should be possible to run backdraw on other Unix platforms with X Windows
and OSF/Motif libraries available.
.SH OPTIONS
.I Backdraw
accepts all of the standard X Toolkit command line options along with the 
additional options listed below:
.TP 8
.B \-drawfont <font>
This option specifies the default font used to draw strings.
.TP 8
.B \-help
This option indicates that a summary of the allowed
command line options should be
printed on the standard error.
.TP 8
.B \-/+interactive
Turns on/off interactive mode. If interactive mode is enabled, each drawing
command results in redrawing of the drawing area. If interactive mode is
disabled, nothing will be drawn until the \fIdraw-all-planes\fP
command is received.
.TP 8
.B \-/+planeGC
Turns on/off the usage of different graphics context for every plane (see
description).
.TP 8
.B \-planes <number>
Specifies the number of planes that are used. Software emulation makes the
use of an unlimited number of planes possible. However, the usage of a large
number of planes decreases the graphics performance and is also not
considered very useful. Therefore, this number is limited by 32.
.TP 8
.B \-plotdir <directory name>
Specifies the name of the directory in which plot files are created and
written by backdraw. Currenty, a plot is saved in the file \fIbackdraw.ps\fP.
The default directory is /tmp. If a directory is specified which does not
exist or a directory which is not writable, the default directory is used
in stead of the specified directory.
.TP 8
.B \-update <number>
Specifies the update time interval (in seconds). This options turns the
interactive mode off and updates the picture each <number> seconds.
.TP 8
.B \-/+verbose
Turn on/off verbose mode. If verbose mode is turned on, the commands that
are not recognized are reported on standard error.
.SH COMMANDS
The following commands are accepted by the server:
.TP 8
\fBclear-plane\fP \fI<plane_no>\fP
Clears the plane identified with \fIplane_no\fP and redraws the contents of
the drawing area.
.TP 8
\fBclear-planes\fP
Clears all planes and redraws the contents of
the drawing area.
.TP 8
\fBdraw-all-planes\fP
Draws the picture in the drawing area. This command only works in
non-interactive mode.
.TP 8
\fBdraw-circle\fP \fI<x> <y> <radius>\fP
Adds a circle to the picture with center \fI(x,y)\fP and specified radius.
.TP 8
\fBdraw-line\fP \fI<x1> <y1> <x2> <y2>\fP
Adds a line to the picture between the specified coordinates.
.TP 8
\fBdraw-point\fP \fI<x> <y>\fP
Adds a point to the picture. Points are translated to a single pixel on the
screen. Its size will not change if the view port of the picture changes.
.TP 8
\fBdraw-rectangle\fP \fI<x> <y> <width> <height>\fP
Adds a rectangle to the picture.
.TP 8
\fBdraw-string\fP \fI<x> <y> <quoted-string>\fP
Adds a string at the specified position to the database. This position is
the position is mapped on the default font position (lowerleft corner). A
quoted string is any number of printable characters enclosed between double
quotes.
.TP 8
\fBfill-circle\fP \fI<x> <y> <radius>\fP
Adds a filled circle to the picture
with center \fI(x,y)\fP and specified radius.
.TP 8
\fBfill-rectangle\fP \fI<x> <y> <width> <height>\fP
Adds a filled rectangle to the picture.
.TP 8
\fBinteractive\fP \fI<flag>\fP
Turns on/off interactive mode, if the flag specified is non-zero/zero. This
command is only effective, if time interval updating is turned off.
.TP 8
\fBset-color\fP \fI<colorname>\fP
Sets the color of the current graphics context to \fIcolorname\fP.
\fIcolorname\fP is either a named color in the X color database or a color
specified in the # formats.
.TP 8
\fBset-font\fP \fI<fontname>\fP
Sets the font of the current graphics context to \fIfontname\fP.
.TP 8
\fBset-pixmap\fP \fI<max. plane>\fP
Puts the contents of the lower planes [0,\fI<max. plane>\fP] in a pixmap. This
command improves efficiency significantly, if the contents of the lower
planes is used to show the background of the picture and the contents of
the upper planes is cleared very often, which results in drawing
the contents of the lower planes again.
.TP 8
\fBset-plane\fP \fI<plane>\fP
Sets the current plane to \fIplane\fP.
.TP 8
\fBset-view-port\fP \fI<x> <y> <width> <height>\fP
Sets the view port to the area specified by the arguments.
.TP 8
\fBset-world\fP \fI<x> <y> <width> <height>\fP
Sets a bounding box for the picture. This bounding box specifies the area
that will be displayed after executing a world command from the \fIView\fP
pulldown menu.
.TP 8
\fBquit\fP
Quits backdraw.
.SH WIDGETS
In order to specify resources, it is useful to know the hierarchy of
the widgets which compose \fIbackdraw\fR.  In the notation below,
indentation indicates hierarchical structure.  The widget class name
is given first, followed by the widget instance name.
.sp
.nf
.ta .5i 1.0i 1.5i 2.0i 2.5i 3.0i 3.5i 4.0i 4.5i
Backdraw	backdraw
	MainWindow	container
	SeparatorGadget	MainWinSep1
	SeparatorGadget	MainWinSep2
	SeparatorGadget	MainWinSep3
	RowColumn	Menubar
		CascadeButton	cascade
		CascadeButton	cascade
		MenuShell	popup_File
			RowColumn	File
				PushButton	quit
			RowColumn	View
				PushButton	zoomIn
				PushButton	zoomOut
	DrawingArea	canvas
.fi
.sp
.SH "APPLICATION RESOURCES"
\fIbackdraw\fP has the following application-specific resources which allow
customizations unique to \fIbackdraw\fP.
.PP
.TP 18
\fBdrawfont\fP (Class \fBDrawfont\fP)
Specifies the default font used to string objects.
.TP 18
\fBhelp\fP (Class \fBHelp\fP)
Specifies whether or not help information is displayed on the standard
error stream. This resource is usually controlled through a command line
option.
.TP 18
\fBinteractive\fP (Class \fBInteractive\fP)
Specifies whether or not the interactive mode is enabled.
.TP 18
\fBplaneGC\fP (Class \fBPlaneGC\fP)
Specifies whether or not a unique graphics context is used for each plane.
.TP 18
\fBplanes\fP (Class \fBPlanes\fP)
The number of planes that can be used during the session.
.TP 18
\fBupdate\fP (Class \fBUpdate\fP)
Specifies the time interval between drawing area updates. It is only
effictive, if the number specified is positive. In that case, interactive
mode is disabled.
.TP 18
\fBverbose\fP (Class \fBVerbose\fP)
Specifies whether or not backdraw reports syntax errors on standard error.
.SH ENVIRONMENT
.PP
.TP 8
.B DISPLAY
the default host and display number.
.TP 8
.B XENVIRONMENT
the name of a resource file that overrides the global resources
stored in the RESOURCE_MANAGER property.
.SH FILES
/usr/lib/X11/app-defaults/Backdraw - specifies required resources
.SH "SEE ALSO"
.SH BUGS
.I Backdraw
\fBshould be\fP bug free. Please report bugs and inconsistencies to the author.
Also suggestions can be reported to the author.
.SH COPYRIGHT
Copyright 1992
.br
All Rights Reserved.
.br
Hans Fleurkens.
.SH AUTHOR
Hans Fleurkens
.br
Design Automation Section
.br
Eindhoven University of Technology
.br
The Netherlands
.br
Phone: +31-40-473238
.br
Fax: +31-40-464527
.br
E-mail: hans@es.ele.tue.nl
