#include <stdio.h>
#include <Xm/Xm.h>


#include "defines.h"
#include "types.h"
#include "globals.h"

extern DRAWSTATUS drawstatus;
extern SBOX viewport;


void draw_object (DRAWOBJ *obj, GCDATA *gcdata, Drawable xid)
{
  XGCValues values;
  unsigned long mask;

  if (gcdata->gc_pixel != obj->pixel) {
    values.foreground = obj->pixel;
    mask = GCForeground;

    XChangeGC (XtDisplay (canvas), gcdata->gc, mask, &values);
    gcdata->gc_pixel = obj->pixel;
  }
  (*obj->drawfunc) (obj, gcdata, xid);
}


void draw_line (DRAWOBJ *obj, GCDATA *gcdata, Drawable xid)
{
  SBOX *sbox = obj->sbox;
  LINE *line = obj->object;
  int sx, sy, sw, sh;
  GC gc = gcdata->gc;
  
  sx = (line->x1 - viewport.x) / scaling + 0.5;
  sy = (line->y1 - viewport.y) / scaling + 0.5;
  sw = (line->x2 - viewport.x) / scaling + 0.5;
  sh = (line->y2 - viewport.y) / scaling + 0.5;


#ifdef DEBUG
  fprintf (stderr, "scaling: %f\n", scaling);
  fprintf (stderr, "viewport: %d %d %d %d\n",
	   viewport.x, viewport.y, viewport.width, viewport.height);
  fprintf (stderr, "draw_line: %d, %d, %d, %d\n", sx, sy, sw, sh);
#endif
  
  XDrawLine (XtDisplay (canvas), xid,
	     gc, sx, sy, sw, sh);
}


void draw_point (DRAWOBJ *obj, GCDATA *gcdata, Drawable xid)
{
  SBOX *sbox = obj->sbox;
  GC gc = gcdata->gc;
  int sx, sy;

  
  sx = (sbox->x - viewport.x) / scaling + 0.5;
  sy = (sbox->y - viewport.y) / scaling + 0.5;

  XDrawPoint (XtDisplay (canvas), xid,
	      gc, sx, sy);
}


void draw_rectangle (DRAWOBJ *obj, GCDATA *gcdata, Drawable xid)
{
  RECT *rect = obj->object;
  SBOX *sbox = obj->sbox;
  GC gc = gcdata->gc;
  int sx, sy, sw, sh;

  
  sx = (sbox->x - viewport.x) / scaling + 0.5;
  sy = (sbox->y - viewport.y) / scaling + 0.5;
  sw = (sbox->x + sbox->width - viewport.x) / scaling + 0.5;
  sh = (sbox->y + sbox->height - viewport.y) / scaling + 0.5;

#ifdef DEBUG
  fprintf (stderr, "scaling: %f\n", scaling);
  fprintf (stderr, "viewport: %d %d %d %d\n",
	   viewport.x, viewport.y, viewport.width, viewport.height);
  fprintf (stderr, "draw_rectangle: %d, %d, %d, %d\n", sx, sy, sw, sh);
#endif

  if ((sw == 0) && (sh == 0)) {
    XDrawPoint (XtDisplay (canvas), xid, gc, sx, sy);
    return;
  }
  if (sh == 0) {
    XDrawLine (XtDisplay (canvas), xid,
	       gc, sx, sy, sw, sy);
    return;
  }
  if (sw == 0) {
    XDrawLine (XtDisplay (canvas), xid,
	       gc, sx, sy, sx, sh);
    return;
  }
  if (rect->fill)
    XFillRectangle (XtDisplay (canvas), xid,
		    gc, sx, sy, sw - sx, sh - sy);
  else
    XDrawRectangle (XtDisplay (canvas), xid,
		    gc, sx, sy, sw - sx, sh - sy);
}


void draw_circle (DRAWOBJ *obj, GCDATA *gcdata, Drawable xid)
{
  GC gc = gcdata->gc;
  CIRCLE *circle = obj->object;
  SBOX *sbox = obj->sbox;
  int sx, sy, sw, sh;

  
  sx = (sbox->x - viewport.x) / scaling + 0.5;
  sy = (sbox->y - viewport.y) / scaling + 0.5;
  sw = sbox->width / scaling + 0.5;
  sh = sbox->height / scaling + 0.5;

#ifdef DEBUG
  fprintf (stderr, "scaling: %f\n", scaling);
  fprintf (stderr, "viewport: %d %d %d %d\n",
	   viewport.x, viewport.y, viewport.width, viewport.height);
  fprintf (stderr, "draw_circle: %d, %d, %d, %d\n", sx, sy, sw, sh);
#endif
  
  if (circle->fill)
    XFillArc (XtDisplay (canvas), xid,
		    gc, sx, sy, sw, sh, 0, 23040);
  else
    XDrawArc (XtDisplay (canvas), xid,
		    gc, sx, sy, sw, sh, 0, 23040);
}


void draw_string (DRAWOBJ *obj, GCDATA *gcdata, Drawable xid)
{
  GC gc = gcdata->gc;
  STRING *dstr = obj->object;
  SBOX *sbox = obj->sbox;
  int sx, sy, sw, sh;
  
  sx = (dstr->x - viewport.x) / scaling + 0.5;
  sy = (dstr->y - viewport.y) / scaling + 0.5;

  if (gcdata->gc_font != dstr->font) {
    XSetFont (XtDisplay (canvas), gc, dstr->font->fid);
    gcdata->gc_font = dstr->font;
  }
  
  XDrawString (XtDisplay (canvas), xid,
	       gc, sx, sy, dstr->str, strlen (dstr->str));

#ifdef COMMENT
  sx = (sbox->x - viewport.x) / scaling + 0.5;
  sy = (sbox->y - viewport.y) / scaling + 0.5;
  sw = sbox->width / scaling + 0.5;
  sh = sbox->height / scaling + 0.5;
  XDrawRectangle (XtDisplay (canvas), xid,
		  gc, sx, sy, sw, sh);
#endif
}


