/* 
 * file-documentation
 * =========================================================================
 * error.c -- Error routines
 * 
 * Author          : Hans Fleurkens
 * Created On      : Thu Jun 14 08:47:57 1990
 * Last Modified By: Hans Fleurkens
 * Last Modified On: Tue Sep 28 09:33:22 1993
 * Update Count    : 46
 * Status          : ok
 * 
 * History
 * PURPOSE
 * 	Defines a general error routine for allocation routines.
 * 	Also defines a pointer for allocation routines.
 * =========================================================================
 * Copyright (c) 1990-1993
 * All Rights Reserved
 * Hans Fleurkens
 * Eindhoven University of Technology
 * The Netherlands
 * =========================================================================
 */

#define USE_X /* define USE_X, iff OSF/Motif is used */
#define _ERROR_C

#include <stdio.h>

#include "error.h"

#ifdef USE_X /* Include files for OSF/Motif */
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#endif

/* Temporary variable for ALLOC macros, see alloc.h: */
void *calloc_ptr;
char *str_ptr;

/* buffer used for error messages */
char errbuf[ERRBUF_SIZE];

#ifdef USE_X
static Arg al[10];
static int ac;
static Widget msgDialog;
static int eventHandlingEnabled = 0;
static int msgHandlerMapped = 0;

static Widget CreateMsgDialog ();
#endif

static void _defaultError ();
static void _defaultWarning ();
static void _defaultMsg ();

typedef void (*ErrorHandlerProc) ();

static ErrorHandlerProc errorHandler = _defaultError;
static ErrorHandlerProc warningHandler = _defaultWarning;
static ErrorHandlerProc msgHandler = _defaultMsg;

void errorMsg (message)
     char *message;
{
  (*errorHandler) (message);
}

void warningMsg (message)
     char *message;
{
  (*warningHandler) (message);
}

void infoMsg (message)
     char *message;
{
  (*msgHandler) (message);
}


static void _defaultError (message)
     char *message;
{
  fprintf (stderr, "Error: %s\n", message);
  exit(1);
}

static void _defaultWarning (message)
     char *message;
{
  fprintf (stderr, "Warning: %s\n", message);
}

static void _defaultMsg (message)
     char *message;
{
#ifdef USE_X
  XmString str, oldstr;
  char *rstr;
  
  if (!eventHandlingEnabled) {
    fprintf (stderr, "%s\n", message);
  }
  else {
    /* map window if necessary */
    if (!msgHandlerMapped) {
      XtManageChild (msgDialog);
      msgHandlerMapped = 1;
    }
    /* retrieve old message from resource */
    ac = 0;
    XtSetArg (al[ac], XmNmessageString, &oldstr); ac++;
    XtGetValues (msgDialog, al, ac);

    /* put message into dialog window */
    ac = 0;
    str = XmStringCreate (message, XmSTRING_DEFAULT_CHARSET);
    XtSetArg (al[ac], XmNmessageString, str); ac++;
    XtSetValues (msgDialog, al, ac);
    XmStringFree (str);

    XmStringGetLtoR (oldstr, XmSTRING_DEFAULT_CHARSET, &rstr);
#ifdef COMMENT
    fprintf (stderr, "Old string retrieved : %s\n", rstr);
#endif
    XmStringFree (oldstr);
    free (rstr);
    XmUpdateDisplay (msgDialog);
  }
  
#else
  
  fprintf (stderr, "%s\n", message);
  
#endif
  
}


#ifdef USE_X

void InitErrorHandling (parent)
     Widget parent;
{
  eventHandlingEnabled = 1;
  msgHandlerMapped = 0;
  msgDialog = CreateMsgDialog (parent);
}


static void unmap_CB (widget, client, call)
     Widget widget;
     caddr_t client;
     caddr_t call;
{
  msgHandlerMapped = 0;
}


static Widget CreateMsgDialog (parent)
     Widget parent;
{
  Widget msgbox;
  Widget child;
  
  ac = 0;
  XtSetArg (al[ac], XmNresizePolicy, XmRESIZE_GROW); ac++;
  msgbox = XmCreateInformationDialog (parent, "msgDialog", al, ac);

  child = XmMessageBoxGetChild (msgbox, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild(child);
  child = XmMessageBoxGetChild (msgbox, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild(child);

  child = XmMessageBoxGetChild (msgbox, XmDIALOG_OK_BUTTON);
  XtAddCallback(child, XmNactivateCallback, (XtCallbackProc) unmap_CB, NULL);

  return msgbox;
}

#endif


