/* 
 * file-documentation
 * =========================================================================
 * get_string.c -- Get string dialog
 * 
 * Author          : Hans Fleurkens
 * Created On      : Fri Sep 13 10:03:19 1991
 * Last Modified By: Hans Fleurkens
 * Last Modified On: Tue Sep 28 09:31:40 1993
 * Update Count    : 37
 * Status          : ansi; ok
 * 
 * History
 * Description:
 * 	Routines that prompt for a string and process it.
 * =========================================================================
 * Copyright (c) 1990-1993
 * All Rights Reserved
 * Hans Fleurkens
 * Eindhoven University of Technology
 * The Netherlands
 * =========================================================================
 */


#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/SelectioB.h>


#include "alloc.h"
#include "defines.h"
#include "types.h"
#include "globals.h"


void get_string_handler (PROMPT_DATA *data)
{
  XmString str;
  Widget text;
  
  ac = 0;  
  str = XmStringCreate (data->question_str, XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNselectionLabelString, str); ac++;
  XtSetValues (data->prompt_dialog, al, ac);
  XmStringFree (str);

  text = XmSelectionBoxGetChild (data->prompt_dialog, XmDIALOG_TEXT);
  XmTextSetString (text, data->default_str);

  XtManageChild (data->prompt_dialog);
}


static void ok_cb (Widget widget,
		   PROMPT_DATA *prompt_data,
		   XmSelectionBoxCallbackStruct *cb_data)
{
  char *ret_str;
  
  XmStringGetLtoR (cb_data->value, XmSTRING_DEFAULT_CHARSET, &ret_str); 
  if (prompt_data->ok_func)
    (*prompt_data->ok_func) (ret_str, prompt_data->client_data);

  FREE (ret_str);
}

static void cancel_cb (Widget widget,
		       PROMPT_DATA *prompt_data,
		       XmSelectionBoxCallbackStruct *cb_data)
{
  char *ret_str;
  
  XmStringGetLtoR (cb_data->value, XmSTRING_DEFAULT_CHARSET, &ret_str); 
  if (prompt_data->cancel_func)
    (*prompt_data->cancel_func) (ret_str, prompt_data->client_data);
  else
    XtUnmanageChild (widget);
  FREE (ret_str);
}  

  
void CreateGetStringDialog (Widget parent, PROMPT_DATA *prompt_data)
{
  XmString str1;
  Widget dialog;
  Widget child;
  
  ac = 0;
  str1 = XmStringCreate ("Get String Dialog", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNautoUnmanage, False); ac++;
  XtSetArg (al[ac], XmNdialogTitle, str1); ac++;
  dialog = XmCreatePromptDialog (parent, "getString", al, ac);
  XmStringFree (str1);

  child = XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (child);

  /*
   * =======================================================================
   * Watch Out! (see XtAddCallback)
   * To prevent warnings here, we cast the callbacks to type
   * XtCallbackProc; this goes around the checks of the ansi C compiler
   * =======================================================================
   */  
  XtAddCallback (dialog, XmNokCallback,
		 (XtCallbackProc)ok_cb, prompt_data);
  XtAddCallback (dialog, XmNcancelCallback,
		 (XtCallbackProc)cancel_cb, prompt_data);

  prompt_data->prompt_dialog = dialog;
  
}





