/* 
 * file-documentation
 * =========================================================================
 * handler.c -- Event handlers
 * 
 * Author          : Hans Fleurkens
 * Created On      : Fri Jul 31 14:56:50 1992
 * Last Modified By: Hans Fleurkens
 * Last Modified On: Wed Jan 27 20:48:56 1993
 * Update Count    : 35
 * Status          : none
 * 
 * History
 * Description:
 * 	>
 * =========================================================================
 * Copyright (c) 1990-1992
 * All Rights Reserved
 * Hans Fleurkens
 * Eindhoven University of Technology
 * The Netherlands
 * =========================================================================
 */

#include <stdio.h>
#include <Xm/Xm.h>

#include "defines.h"
#include "types.h"
#include "globals.h"

extern SBOX viewport;
extern SBOX windowport;

extern void press_rubband_box (Widget, XtPointer, XButtonEvent *, Boolean *);
extern void track_rubband_box (Widget, XtPointer, XButtonEvent *, Boolean *);
extern void release_rubband_box (Widget, XtPointer, XButtonEvent *, Boolean *);

static int nr_ehandlers;
static EHANDLER *ehandlers;

void install_handlers (EHANDLER *list, int no)
{
  int i;
  
  ehandlers = list;
  nr_ehandlers = no;
  for (i = 0; i < no; i++)
    XtInsertEventHandler (canvas,
			  ehandlers[i].event_mask,
			  ehandlers[i].nonmaskable,
			  ehandlers[i].proc,
			  ehandlers[i].client_data,
			  ehandlers[i].position);
}

void uninstall_handlers ()
{
  int i;

  for (i = 0; i < nr_ehandlers; i++)
    XtRemoveEventHandler (canvas,
			  ehandlers[i].event_mask,
			  ehandlers[i].nonmaskable,
			  ehandlers[i].proc,
			  ehandlers[i].client_data);
  nr_ehandlers = 0;
  ehandlers = NULL;
}

/* 
 * =========================================================================
 * Callbacks installing new event handlers
 * =========================================================================
 */

#define CENTER_HANDLERS 1

static void center_handler (Widget, XtPointer, XButtonEvent *, Boolean *);

EHANDLER center_handlers[] = {
  { ButtonPressMask, False, (XtEventHandler)center_handler, NULL, XtListHead },
};
  
void center_cb (Widget widget, XtPointer client_data, XtPointer call_data)
{
  uninstall_handlers ();
  install_handlers (center_handlers, CENTER_HANDLERS);
}


static void center_handler (Widget widget,
			    XtPointer client_data,
			    XButtonEvent  *event,
			    Boolean *cont_to_disp)
{

  viewport.x += (event->x - windowport.width / 2) *  scaling;
  viewport.y += (event->y - windowport.height / 2) *  scaling;

  if (use_pixmap >= 0)
    init_pixmap (pixmap, use_pixmap, pix_width, pix_height);

  XClearArea (XtDisplay (canvas), XtWindow (canvas), 0, 0, 0, 0, TRUE);
}


#define ZOOMAREA_HANDLERS 4

static void zoomarea_handler (Widget, XtPointer, XButtonEvent *, Boolean *);

EHANDLER zoomarea_handlers[] = {
  {ButtonPressMask,False,(XtEventHandler)press_rubband_box,NULL,XtListHead},
  {ButtonMotionMask,False,(XtEventHandler)track_rubband_box,NULL,XtListHead},
  {ButtonReleaseMask,False,(XtEventHandler)zoomarea_handler,NULL,XtListHead},
  {ButtonReleaseMask,False,(XtEventHandler)release_rubband_box,NULL,XtListHead},
};
  
void zoom_area_cb (Widget widget, XtPointer client_data, XtPointer call_data)
{
  uninstall_handlers ();
  install_handlers (zoomarea_handlers, ZOOMAREA_HANDLERS);
}

static void zoomarea_handler (Widget widget,
			      XtPointer client_data,
			      XButtonEvent  *event,
			      Boolean *cont_to_disp)
{
  float div_width, div_height;

  if ((rubboxdata.x1 == rubboxdata.x2) ||
      (rubboxdata.y1 == rubboxdata.y2)) {
    if (appdata.verbose == True)
      fprintf (stderr, "Box specifying area too small to perform zoom\n");
    return;
  }

  viewport.x += MIN (rubboxdata.x1, rubboxdata.x2) * scaling;
  viewport.y += MIN (rubboxdata.y1, rubboxdata.y2) * scaling;
  viewport.width = ABS (rubboxdata.x1 - rubboxdata.x2) * scaling;
  viewport.height = ABS (rubboxdata.y1 - rubboxdata.y2) * scaling;

  div_width = (float)viewport.width / windowport.width;
  div_height = (float)viewport.height / windowport.height;
  scaling = MAX (div_width, div_height);

  if (use_pixmap >= 0)
    init_pixmap (pixmap, use_pixmap, pix_width, pix_height);

  XClearArea (XtDisplay (canvas), XtWindow (canvas), 0, 0, 0, 0, TRUE);
}
