/* 
 * file-documentation
 * =========================================================================
 * main.c -- main ()
 * 
 * Author          : Hans Fleurkens
 * Created On      : Tue Jan 28 15:24:14 1992
 * Last Modified By: Hans Fleurkens
 * Last Modified On: Tue Sep 28 09:30:21 1993
 * Update Count    : 313
 * Status          : none
 * 
 * History
 * Description:
 * 	>
 * =========================================================================
 * Copyright (c) 1992-1993
 * All Rights Reserved
 * Hans Fleurkens
 * Eindhoven University of Technology
 * The Netherlands
 * =========================================================================
 */

#include <stdio.h>
#include <ctype.h>
#include <sys/file.h>

#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/MainW.h>

#define _MAIN_C

#include "defines.h"
#include "types.h"
#include "globals.h"
#include "alloc.h"

extern Widget CreateMenuBar ();

PROMPT_DATA get_string_data;
XtAppContext app;
SBOX windowport;
SBOX viewport;
SBOX defworld = { 0, 0, -1, -1 };
SBOX world = { 0, 0, -1, -1 };
DRAWSTATUS drawstatus;
PLANE *planes;
GCDATA default_gc;
GC rubbergc;
GC fillgc;

static XColor xcolor_fg;

static String fallback_resources[] = {
  "*geometry: 200x200",
  "*workArea*background: white",
  NULL
};

static XrmOptionDescRec options[] = {
  { "-drawfont", "*drawfont",  XrmoptionSepArg, NULL },
  { "-help", "*help", XrmoptionNoArg, "True" },
  { "-interactive", "*interactive", XrmoptionNoArg, "True" },
  { "+interactive", "*interactive", XrmoptionNoArg, "False" },
  { "-planeGC", "*planeGC", XrmoptionNoArg, "True" },
  { "+planeGC", "*planeGC", XrmoptionNoArg, "False" },
  { "-planes", "*planes",  XrmoptionSepArg, NULL },
  { "-plotdir", "*plotdir",  XrmoptionSepArg, NULL },
  { "-update", "*update", XrmoptionSepArg, "0" },
  { "-verbose", "*verbose", XrmoptionNoArg, "True" },
  { "+verbose", "*verbose", XrmoptionNoArg, "False" },
};

static XtResource resources[] = {
  {"help", "Help", XtRBool, sizeof(Bool),
     XtOffset(ApplicationDataPtr, help), XtRString, "False"},
  {"verbose", "Verbose", XtRBool, sizeof(Bool),
     XtOffset(ApplicationDataPtr, verbose), XtRString, "False"},
  {"planes", "Planes", XtRInt, sizeof (int),
     XtOffset(ApplicationDataPtr, nr_planes), XtRString, "1"},
  {"drawfont", "Drawfont", XtRFontStruct, sizeof(XFontStruct *),
     XtOffset(ApplicationDataPtr, drawfont), 
     XtRString, "9x15" },
  {"planeGC", "PlaneGC", XtRBool, sizeof(Bool),
     XtOffset(ApplicationDataPtr, plane_gc), XtRString, "False"},
  {"plotdir", "Plotdir", XtRString, sizeof(String),
     XtOffset(ApplicationDataPtr, plot_dir), XtRString, "/tmp"},
  {"interactive", "Interactive", XtRBool, sizeof(Bool),
     XtOffset(ApplicationDataPtr, interactive), XtRString, "True"},
  {"update", "Update", XtRInt, sizeof(int),
     XtOffset(ApplicationDataPtr, update), XtRString, "0"},
};


static first_expose = 0;


static void init_drawstatus ()
{
  drawstatus.plane = 0;
}


static char *help_options[] = {
  "Usage: escape [options]",
  "where options include:",
  "   -help                  shows this text",
  "   -display <name>        Name of X server to connect to",
  "   -iconic                Start in iconic state",
  "   -geometry <geometry>   Specifies the geometry of the application",
  "   -name <name>           Change the name used for retrieving resources",
  "   -title <name>          Change the name without affecting resources",
  "   -xrm <resource>        Specifies a resource",
  "   -bordercolor <color>   Specifies the window border color",
  "   -bd <color>            Specifies the window border color",
  "   -borderwidth <color>   Specifies the window border width",
  "   -bw <color>            Specifies the window border width",
  "   -foreground <color>    Specifies the foreground color",
  "   -fg <color>            Specifies the foreground color",
  "   -background <color>    Specifies the background color",
  "   -bg <color>            Specifies the background color",
  "   -font <font>           Specifies the default font",
  "   -fn <font>             Specifies the default font",
  "   -drawfont <font>       Specifies the default font to draw string objects",
  "   -/+planeGC             Enables/disables the usage of unique GCs for every plane",
  "   -planes <number>       Specifies the number of planes",
  "   -plotdir <dirname>     Specifies the directory to store plot files\n",
  "   -/+verbose             Show/suppress (error) messages from backdraw\n",
  NULL
};


static void print_usage ()
{
  char **hp;
  for (hp = help_options; *hp; hp++)
    fprintf (stderr, "%s\n", *hp);
}

static void create_fill_gc (Widget w)
{
  XGCValues values;
  unsigned long valuemask;
  Pixel bg;

  ac = 0;
  XtSetArg (al[ac], XtNbackground, &bg); ac++;
  XtGetValues (w, al, ac);

  values.function = GXcopy;
  values.background = bg;
  values.foreground = bg;

  valuemask = GCForeground | GCBackground | GCFunction;

  fillgc = XCreateGC (XtDisplay(w), XtWindow(w), valuemask, &values);
}


static void create_rubber_gc (Widget w)
{
  XGCValues values;
  unsigned long valuemask;
  Pixel bg, fg;

  ac = 0;
  XtSetArg (al[ac], XtNbackground, &bg); ac++;
  XtSetArg (al[ac], XtNforeground, &fg); ac++;
  XtGetValues (w, al, ac);

  values.line_style = LineDoubleDash;
  values.function = GXxor;
  values.background = fg;
  values.foreground = fg ^ bg;

  valuemask = GCForeground | GCBackground | GCFunction | GCLineStyle;

  rubbergc = XCreateGC (XtDisplay(w), XtWindow(w), valuemask, &values);
}


static void create_gc (Widget drawarea, GCDATA *gcdata)
{
  XGCValues values;
  unsigned long mask;
  Pixel bg, fg;

  ac = 0;
  XtSetArg (al[ac], XtNbackground, &bg); ac++;
  XtSetArg (al[ac], XtNforeground, &fg); ac++;
  XtGetValues (drawarea, al, ac);

  values.background = bg;
  values.foreground = fg;
  values.font = appdata.drawfont->fid;
  mask = GCForeground | GCBackground | GCFont;

  gcdata->gc = XCreateGC (XtDisplay (drawarea), XtWindow (drawarea),
			     mask, &values);
  gcdata->def_pixel = fg;
  gcdata->gc_pixel = fg;
  gcdata->red = xcolor_fg.red;
  gcdata->green = xcolor_fg.green;
  gcdata->blue = xcolor_fg.blue;
  gcdata->def_font = appdata.drawfont;
  gcdata->gc_font = appdata.drawfont;
}


void restore_gc (GCDATA *gcdata)
{
  XGCValues values;
  unsigned long mask = 0;

  if (gcdata->def_pixel != gcdata->gc_pixel) {
    values.foreground = gcdata->def_pixel;
    mask |= GCForeground;
    gcdata->gc_pixel = gcdata->def_pixel;
  }
  if (gcdata->def_font != gcdata->gc_font) {
    values.font = gcdata->def_font->fid;
    mask |= GCFont;
    gcdata->gc_font = gcdata->def_font;
  }
  if (mask != 0)
    XChangeGC (XtDisplay (canvas), gcdata->gc, mask, &values);
}

static void parse_msg (buf)
     char *buf;
{
  char *bufptr;
  char token[BUFSIZ];

  if (sscanf (buf, "%s", token) != 1) {
#ifdef DEBUG
    fprintf (stderr, "Warning: No token found\n");
#endif
    return;
  }
  bufptr = buf + strlen(token);
  parse_token_and_exec (token, bufptr);
}

static void read_input (client_data, fd, id)
     XtPointer client_data;
     int *fd;
     XtInputId *id;
{
  int i;
  int len;
  int x,y;
  int ptr;
  char buf[BUFSIZ];


#ifdef COMMENT /* gets () improves the performance by a factor of six */
  ptr = 0;
  while (read (*fd, &buf[ptr], 1) == 1) {
    if (buf[ptr] == '\n') {
      buf[ptr] = '\0';
      if (buf[0] != '#')
	parse_msg (buf);
      return;
    }
    ptr++;
  }
#endif
  if (gets (buf))
    parse_msg (buf);
  else
    XtRemoveInput (*id);
}


void clear_canvas()
{
  XClearWindow(XtDisplay (canvas), XtWindow (canvas));
}


static void resize (widget, closure, cb_data)
     Widget widget;
     XtPointer closure;
     XmDrawingAreaCallbackStruct *cb_data;
{
  Dimension width, height; 
  float div, div_width, div_height;

  if (first_expose == 0)
    return;

  /* get window size */
  ac = 0;
  XtSetArg (al[ac], XmNwidth, &width); ac++;
  XtSetArg (al[ac], XmNheight, &height); ac++;
  XtGetValues (canvas, al, ac);

  div_width = (float)viewport.width / width;
  div_height = (float)viewport.height / height;
  div = MAX (div_width, div_height);

  scaling = div;
  
  /* new windowport */
  windowport.width = width;
  windowport.height = height;

  resize_string_boxes (scaling);
  
  if (use_pixmap >= 0) {
    /* pixmap created (and deleted) in parser */
    XFreePixmap (XtDisplay (canvas), pixmap);
    pixmap = XCreatePixmap (XtDisplay (canvas), XtWindow (canvas),
			    width, height, pix_depth);
    init_pixmap (pixmap, use_pixmap, width, height);
    pix_width = width;
    pix_height = height;
  }
  /* show the same, adjust scaling and viewport */
  XClearArea (XtDisplay (canvas), XtWindow (canvas), 0, 0, 0, 0, TRUE);
}

static Widget toplevel; /* shell widget */

static void redraw (widget, closure, cb_data)
     Widget widget;
     XtPointer closure;
     XmDrawingAreaCallbackStruct *cb_data;
{
  Dimension width, height;
  XExposeEvent *xexp;
  static SBOX expbox, newbox;
  
  if (!first_expose) {
    XtAppAddInput (app, 0, (XtPointer)XtInputReadMask, read_input, NULL);
    /* get window size */
    ac = 0;
    XtSetArg (al[ac], XmNwidth, &width); ac++;
    XtSetArg (al[ac], XmNheight, &height); ac++;
    XtGetValues (canvas, al, ac);

    windowport.width = width;
    windowport.height = height;
    viewport.x = 0;
    viewport.y = 0;
    viewport.width = width;
    viewport.height = height;
    scaling = 1.0;

    expbox.width = -1;
    first_expose = 1;
  }

  xexp = (XExposeEvent *) cb_data->event;
  newbox.x = xexp->x;
  newbox.y = xexp->y;
  newbox.width = xexp->width;
  newbox.height = xexp->height;
  update_sbox (&expbox, &newbox);
  
  if (xexp->count == 0) {
#ifdef DEBUG
    fprintf (stderr, "Redrawing %d %d %d %d\n", expbox.x, expbox.y,
	     expbox.width, expbox.height);
#endif
    expbox.width *= scaling;
    expbox.height *= scaling;
    expbox.x = (expbox.x * scaling) + viewport.x;
    expbox.y = (expbox.y * scaling) + viewport.y;
    redraw_planes (0, &expbox);
    expbox.width = -1;
  }
}


static void timer_update (Widget widget, XtIntervalId *id)
{
  if (first_expose)
    redraw_all_planes ();
  XtAppAddTimeOut (app, (unsigned long)(appdata.update * 1000),
		(XtTimerCallbackProc)   timer_update, NULL);
}


void main (argc, argv)
     unsigned int  argc;
     char          **argv;
{
  Widget mainWin;
  Widget menubar;
  Pixel fg, bg;
  int i, len;
  
  /* set line buffering */
  setvbuf (stdin, NULL, _IOLBF, BUFSIZ);
  setvbuf (stdout, NULL, _IOLBF, BUFSIZ);

  init_globals ();
  
  toplevel = XtAppInitialize (&app, "Backdraw", options, XtNumber (options),
			      (int *)&argc, argv, fallback_resources, NULL, 0);

  XtGetApplicationResources (toplevel, &appdata,
			     resources, XtNumber(resources),
			     NULL, 0);

  if (appdata.help == True) {
    print_usage ();
    exit (0);
  }

  len = strlen (appdata.plot_dir);
  if (*(appdata.plot_dir + len - 1) == '/') {
    *(appdata.plot_dir + len - 1) = '\0';
  }
  /* Check for existence of plot directory */
  if (access (appdata.plot_dir, F_OK) == -1) {
    if (appdata.verbose == True)
      fprintf (stderr, "Plot directory (%s) does not exist\n",
	       appdata.plot_dir);
    appdata.plot_dir = NEW_STRING ("/tmp");
  }
  /* is it writable? */
  if (access (appdata.plot_dir, W_OK) == -1) {
    if (appdata.verbose == True)
      fprintf (stderr, "Plot directory (%s) is not writable!\n",
	       appdata.plot_dir);
    appdata.plot_dir = NEW_STRING ("/tmp");
  }

  init_color_table (XtDisplay (toplevel));
  init_planes (appdata.nr_planes);
  init_drawstatus ();
  init_plot ();
  
  ac = 0;
  mainWin = XmCreateMainWindow (toplevel, "container", al, ac);
  XtManageChild (mainWin);

  menubar = CreateMenuBar (mainWin);
  
  canvas = XmCreateDrawingArea (mainWin, "canvas", al, ac);

  XtAddCallback (canvas, XmNresizeCallback,
		 (XtCallbackProc)resize, NULL);
  XtAddCallback (canvas, XmNexposeCallback,
		 (XtCallbackProc)redraw, NULL);
  XtManageChild (canvas);

  XmMainWindowSetAreas (mainWin, menubar, NULL, NULL, NULL, canvas);

  CreateGetStringDialog (toplevel, &get_string_data);
  XtRealizeWidget (toplevel);

  ac = 0;
  XtSetArg (al[ac], XtNbackground, &bg); ac++;
  XtSetArg (al[ac], XtNforeground, &fg); ac++;
  XtGetValues (canvas, al, ac);

  if (pixel_to_rgb (XtDisplay(canvas), fg, &xcolor_fg)) {
    fprintf (stderr, "Can not convert foreground pixel to rgb color\n");
    exit (1);
  }

  create_rubber_gc (canvas);
  create_fill_gc (canvas);
  if (appdata.plane_gc == False) {
    create_gc (canvas, &default_gc);
    drawstatus.gcdata = &default_gc;
  }
  else {
    for (i = 0; i < appdata.nr_planes; i++)
      create_gc (canvas, planes[i].gcdata);
    drawstatus.gcdata = planes[0].gcdata; /* 0 is default plane */
  }
  init_parse ();
  
  if (appdata.update > 0) {
    appdata.interactive = False;
    XtAppAddTimeOut (app, (unsigned long)(appdata.update * 1000),
		  (XtTimerCallbackProc)   timer_update, NULL);
  }

  XtAppMainLoop (app);
}
