/* 
 * file-documentation
 * =========================================================================
 * menubar.c -- 
 * 
 * Author          : Hans Fleurkens
 * Created On      : Tue Jan 28 15:24:14 1992
 * Last Modified By: Hans Fleurkens
 * Last Modified On: Tue Sep 28 09:26:14 1993
 * Update Count    : 2
 * Status          : none
 * 
 * History
 * Description:
 * 	>
 * =========================================================================
 * Copyright (c) 1992-1993
 * All Rights Reserved
 * Hans Fleurkens
 * Eindhoven University of Technology
 * The Netherlands
 * =========================================================================
 */

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>

#include "defines.h"
#include "types.h"
#include "globals.h"

extern SBOX defworld;
extern SBOX viewport;
extern SBOX windowport;

extern void uninstall_handlers (void);
extern void center_cb (Widget, XtPointer, XtPointer);
extern void zoom_area_cb (Widget, XtPointer, XtPointer);
extern void plot_cb (Widget, XtPointer, XtPointer);

static Widget CreateCascade (label, mnemonic, submenu, parent)
     char    *label,
             mnemonic;
     Widget  submenu,
             parent;

{
  Widget    widget;
  int       n;
  Arg       args[MAX_ARGS];
  XmString  tcs;
  
  n = 0;
  tcs = XmStringLtoRCreate (label, XmSTRING_DEFAULT_CHARSET);
  XtSetArg (args[n], XmNlabelString, tcs); n++;
  XtSetArg (args[n], XmNmnemonic, mnemonic); n++;
  XtSetArg (args[n], XmNsubMenuId, submenu); n++;
  widget = XmCreateCascadeButton (parent, "cascade", args, n);
  XtManageChild (widget);
  XmStringFree (tcs);

  return (widget);
}


static void Quit (w, client_data, call_data)
     Widget  w;
     XtPointer client_data;
     XtPointer call_data;
{
  exit (0);
}

static void zoom (Widget w, float *zoomfactor, XtPointer calldata)
{
  SBOX old;
  float div_width, div_height;

  uninstall_handlers ();
  old = viewport;

  viewport.width = (float)old.width * *zoomfactor;
  viewport.height = (float)old.height * *zoomfactor;
  viewport.x += (old.width - viewport.width) / 2;
  viewport.y += (old.height - viewport.height) / 2;

  div_width = (float)viewport.width / windowport.width;
  div_height = (float)viewport.height / windowport.height;
  scaling = MAX (div_width, div_height);


  if (use_pixmap >= 0)
    init_pixmap (pixmap, use_pixmap, pix_width, pix_height);
  
  XClearArea (XtDisplay (canvas), XtWindow (canvas), 0, 0, 0, 0, TRUE);
}

static void zoom_world (Widget w, XtPointer clientdata, XtPointer calldata)
{
  float div_width, div_height;

  uninstall_handlers ();
  if (defworld.width == -1)
    return;
  viewport = defworld;
  div_width = (float)viewport.width / windowport.width;
  div_height = (float)viewport.height / windowport.height;
  scaling = MAX (div_width, div_height);
  
  if (use_pixmap >= 0)
    init_pixmap (pixmap, use_pixmap, pix_width, pix_height);

  XClearArea (XtDisplay (canvas), XtWindow (canvas), 0, 0, 0, 0, TRUE);
}


Widget CreateMenuBar (parent)
     Widget parent;
{
  XmString str;
  
  Widget menubar;
  Widget filePullDown;
  Widget viewPullDown;
  Widget helpPullDown;
  Widget menubtn;
  Widget btn;
  static float zoomin = ZOOM_SMALLER;
  static float zoomout = ZOOM_LARGER;
  
  ac = 0;
  menubar = XmCreateMenuBar (parent, "Menubar", al, ac);
  XtManageChild (menubar);
  
  ac = 0;
  filePullDown = XmCreatePulldownMenu (menubar, "File", al, ac);
  menubtn = CreateCascade ("File", 'F', filePullDown, menubar);

  ac = 0;
  viewPullDown = XmCreatePulldownMenu (menubar, "View", al, ac);
  menubtn = CreateCascade ("View", 'V', viewPullDown, menubar);

#ifdef COMMENT
  ac = 0;
  helpPullDown = XmCreatePulldownMenu (menubar, "Help", al, ac);
  menubtn = CreateCascade ("Help", 'H', helpPullDown, menubar);

  ac = 0;
  XtSetArg (al[ac], XmNmenuHelpWidget, (XtArgVal)menubtn); ac++;
  XtSetValues (menubar, al, ac);
#endif
  
  /* file pulldown menu */
  
  ac = 0;
  str = XmStringCreate ("Plot", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNlabelString, str); ac++;
  XtSetArg (al[ac], XmNmnemonic, 'P'); ac++;
  btn = XmCreatePushButton (filePullDown, "plot", al, ac);
  XtAddCallback (btn, XmNactivateCallback, plot_cb, NULL);
  XtManageChild (btn);
  XmStringFree (str);

  ac = 0;
  str = XmStringCreate ("Quit", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNlabelString, str); ac++;
  XtSetArg (al[ac], XmNmnemonic, 'Q'); ac++;
  btn = XmCreatePushButton (filePullDown, "quit", al, ac);
  XtAddCallback (btn, XmNactivateCallback, Quit, NULL);
  XtManageChild (btn);
  XmStringFree (str);

  /* view pulldown menu */
  ac = 0;
  str = XmStringCreate ("Center", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNlabelString, str); ac++;
  XtSetArg (al[ac], XmNmnemonic, 'C'); ac++;
  btn = XmCreatePushButton (viewPullDown, "center", al, ac);
  XtAddCallback (btn, XmNactivateCallback, center_cb, NULL);
  XtManageChild (btn);
  XmStringFree (str);

  ac = 0;
  str = XmStringCreate ("Zoom Area", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNlabelString, str); ac++;
  XtSetArg (al[ac], XmNmnemonic, 'A'); ac++;
  btn = XmCreatePushButton (viewPullDown, "zoomArea", al, ac);
  XtAddCallback (btn, XmNactivateCallback, zoom_area_cb, NULL);
  XtManageChild (btn);
  XmStringFree (str);

  ac = 0;
  str = XmStringCreate ("Zoom In", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNlabelString, str); ac++;
  XtSetArg (al[ac], XmNmnemonic, 'I'); ac++;
  btn = XmCreatePushButton (viewPullDown, "zoomIn", al, ac);
  XtAddCallback (btn, XmNactivateCallback,(XtCallbackProc) zoom, &zoomin);
  XtManageChild (btn);
  XmStringFree (str);

  ac = 0;
  str = XmStringCreate ("Zoom Out", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNlabelString, str); ac++;
  XtSetArg (al[ac], XmNmnemonic, 'O'); ac++;
  btn = XmCreatePushButton (viewPullDown, "zoomOut", al, ac);
  XtAddCallback (btn, XmNactivateCallback,(XtCallbackProc) zoom, &zoomout);
  XtManageChild (btn);
  XmStringFree (str);

  ac = 0;
  str = XmStringCreate ("World", XmSTRING_DEFAULT_CHARSET);
  XtSetArg (al[ac], XmNlabelString, str); ac++;
  XtSetArg (al[ac], XmNmnemonic, 'W'); ac++;
  btn = XmCreatePushButton (viewPullDown, "world", al, ac);
  XtAddCallback (btn, XmNactivateCallback, zoom_world, NULL);
  XtManageChild (btn);
  XmStringFree (str);

  return menubar;
}
