/* 
 * file-documentation
 * =========================================================================
 * plot.c -- 
 * 
 * Author          : Hans Fleurkens
 * Created On      : Mon Aug  3 11:10:21 1992
 * Last Modified By: Hans Fleurkens
 * Last Modified On: Fri Sep 17 12:53:42 1993
 * Update Count    : 100
 * Status          : none
 * 
 * History
 * Description:
 * 	>
 * =========================================================================
 * Copyright (c) 1990-1992
 * All Rights Reserved
 * Hans Fleurkens
 * Eindhoven University of Technology
 * The Netherlands
 * =========================================================================
 */

#include <stdio.h>
#include <time.h>
#include <pwd.h>

#include <Xm/Xm.h>

#include "defines.h"
#include "types.h"
#include "globals.h"
#include "alloc.h"
#include "plot.h"

#define BOXOFFSET 5

extern SBOX viewport;
extern PROMPT_DATA get_string_data;

extern get_string_handler (PROMPT_DATA *);

static int plot_device = PLOT_FILE;
static int norm_epsf_x;
static int norm_epsf_y;
static float norm_epsf_scale;

PLOT_OPTIONS plot_options;


void init_plot ()
{
  plot_options.color_ps = True;
  plot_options.norm_epsf = True;
}

static void generate_header (FILE *fp, SBOX *sbox)
{
  struct passwd *my_passwd;
  long clock;
  int llx, lly, urx, ury;
  float nllx, nlly, nurx, nury;
  int bllx, blly, burx, bury;
  
  time (&clock);
  my_passwd = getpwuid (getuid ());
  
  llx = MIN (sbox->x, sbox->x + sbox->width) - BOXOFFSET;
  lly = MIN (-sbox->y, -(sbox->y + sbox->height)) - BOXOFFSET;
  urx = MAX (sbox->x, sbox->x + sbox->width) + BOXOFFSET;
  ury = MAX (-sbox->y, -(sbox->y + sbox->height)) + BOXOFFSET;
  
  if (plot_options.norm_epsf == True) {
    float sx, sy;
    norm_epsf_x = -llx;
    norm_epsf_y = -lly;
    llx += norm_epsf_x;
    lly += norm_epsf_y;
    urx += norm_epsf_x;
    ury += norm_epsf_y;
    sx = urx / 520.0;   /* A4 : 8.27x11.69 inches; 1 inch == 72 pt. */
    sy = ury / 750.0;   /* A4 : 595x841 pt. */
    norm_epsf_scale = MAX (1.0 , MAX (sx, sy));
    nllx = llx / norm_epsf_scale;
    nlly = lly / norm_epsf_scale;
    nurx = urx / norm_epsf_scale + 72;
    nury = ury / norm_epsf_scale + 72;
    norm_epsf_x += 36 * norm_epsf_scale;
    norm_epsf_y += 36 * norm_epsf_scale;
  }
  
  fprintf (fp, "%%!PS-Adobe-2.0 EPSF-2.0\n" );
  if (plot_options.norm_epsf == False)
    fprintf (fp, "%%%%BoundingBox: %d %d %d %d\n", llx, lly, urx, ury);
  else 
    fprintf (fp, "%%%%BoundingBox: %d %d %d %d\n",
	     (int)nllx, (int)nlly, (int)nurx + 1, (int)nury + 1);
  fprintf (fp, "%%%%Title: backdraw plot\n");
  fprintf (fp, "%%%%Creator: %s (%s)\n",
	   my_passwd->pw_name, my_passwd->pw_gecos);
  fprintf (fp, "%%%%CreationDate: %s", ctime (&clock));
  fprintf (fp, "%%%%EndComments\n\n");

  fprintf (fp, "/measuretext {\n");
  fprintf (fp, "	gsave\n");
  fprintf (fp, "	newpath 0 0 moveto\n");
  fprintf (fp, "	true charpath pathbbox\n");
  fprintf (fp, "	4 2 roll 2 copy \n");
  fprintf (fp, "	/stry_pos exch def\n");
  fprintf (fp, "	/strx_pos exch def\n");
  fprintf (fp, "	exch 3 1 roll\n");
  fprintf (fp, "	sub /str_height exch def\n");
  fprintf (fp, "	sub /str_width  exch def\n");
  fprintf (fp, "	grestore\n");
  fprintf (fp, "} def\n\n");
  fprintf (fp, "/l { lineto } bind def\n" ); 
  fprintf (fp, "/m { moveto } bind def\n" );
  fprintf (fp, "/n { newpath } bind def\n" );
  fprintf (fp, "/s { stroke } bind def\n" );
  fprintf (fp, "/c { setrgbcolor } bind def\n" );
  fprintf (fp, "/ff {findfont exch scalefont def} bind def\n" );
  fprintf (fp, "/f { setfont } bind def\n" );
  fprintf (fp, "/t { moveto show } bind def\n" );
  fprintf (fp, "/r { moveto dup stringwidth exch neg exch rmoveto show } bind def\n" );
  fprintf (fp, "/ct { moveto dup measuretext str_width 2 div neg str_height 2 div neg rmoveto show } bind def\n");
  fprintf (fp, "/gs { gsave } bind def\n" );
  fprintf (fp, "/cl { closepath clip } bind def\n" );
  fprintf (fp, "/ge { grestore } bind def\n" );
  fprintf (fp, "%%%%EndProlog\n\n");
  if (plot_options.norm_epsf == False)
    fprintf (fp, "%d %d m\n%d %d l\n%d %d l\n%d %d l\n",
	     llx, lly, llx, ury, urx, ury, urx, lly);
  else
    fprintf (fp, "%.2f %.2f m\n%.2f %.2f l\n%.2f %.2f l\n%.2f %.2f l\n",
	     nllx, nlly, nllx, nury, nurx, nury, nurx, nlly);
  if (plot_options.color_ps == True)
    fprintf (fp, "cl\n0 0 0 setrgbcolor\n" );
  else
    fprintf (fp, "cl\n0 setgray\n" );
  if (plot_options.norm_epsf == True) {
    fprintf (fp, "%f %f scale\n", 1 / norm_epsf_scale, 1 / norm_epsf_scale);
    fprintf (fp, "%d %d translate\n", norm_epsf_x, norm_epsf_y);
  }
  fprintf (fp, "/TEXTFNT 20 /Helvetica ff\n");
  fprintf (fp, "1 setlinewidth\n");
}


static void plot_file (FILE *fp)
{
  norm_epsf_x = 0;
  norm_epsf_y = 0;
  norm_epsf_scale = 800;

  generate_header (fp, &viewport);
  plot_planes (fp, &viewport);
  fprintf (fp, "showpage\n");
}


static void plot_to_file (char *name, XtPointer client_data)
{
  FILE *fp;
  
  if ((fp = fopen (name, "w")) == NULL) {
    if (appdata.verbose == True)
      fprintf (stderr, "Can not open plot file (%s)\n", name);
    return;
  }
  plot_file (fp);
  fclose (fp);
  XtUnmanageChild (get_string_data.prompt_dialog);
}


static void get_plot_file (char *file)
{
  static char question[] = "Save plot file in : ";
  get_string_data.default_str = file;
  get_string_data.question_str = question;
  get_string_data.client_data = NULL;
  get_string_data.ok_func = plot_to_file;
  get_string_data.cancel_func = NULL;
  get_string_handler (&get_string_data);
}

void plot_cb (Widget widget,
	      XtPointer client_data,
	      XtPointer call_data)
{
  static char buf[256];
  FILE *fp;

  switch (plot_device) {
  case PLOT_FILE:
    sprintf (buf, "%s/backdraw.ps", appdata.plot_dir);
    get_plot_file (buf);
    return;
#ifdef COMMENT
  case PLOT_PRINTER:
    confirm_printer (client);
    return;
#endif
  default:
    fprintf (stderr,
	     "WARNING: No output device selected. Plotting aborted\n");
  }
}




void plot_object (DRAWOBJ *obj, FILE *fp)
{
  if (plot_options.color_ps == True) {
    if (plotpixel != obj->pixel) {
      plotpixel = obj->pixel;
      fprintf (fp, "%f %f %f setrgbcolor\n",
	       coltable[plotpixel].red / 65535.0,
	       coltable[plotpixel].green / 65535.0,
	       coltable[plotpixel].blue / 65535.0);
    }
  }
  (*obj->plotfunc) (obj, fp);
}


void plot_line (DRAWOBJ *obj, FILE *fp)
{
  LINE *line = obj->object;

  fprintf (fp, "n %d %d m %d %d l s\n", line->x1, -line->y1,
	   line->x2, -line->y2);
}


void plot_point (DRAWOBJ *obj, FILE *fp)
{
  SBOX *sbox = obj->sbox;

  /* what should I do here? */
}


void plot_rectangle (DRAWOBJ *obj, FILE *fp)
{
  RECT *rect = obj->object;
  SBOX *sbox = obj->sbox;
  int llx, lly, urx, ury;

  
  llx = sbox->x;
  lly = - (sbox->y + sbox->height);
  urx = sbox->x + sbox->width; 
  ury = - sbox->y;

  fprintf (fp, "n %d %d m %d %d l %d %d l %d %d l %d %d l ",
	   llx, lly, llx, ury, urx, ury, urx, lly, llx, lly);
  if (rect->fill)
    fprintf (fp, "fill\n");
  else
    fprintf (fp, "s\n");
}



void plot_circle (DRAWOBJ *obj, FILE *fp)
{
  CIRCLE *circle = obj->object;
  SBOX *sbox = obj->sbox;
  int sx, sy, sw, sh;

  fprintf (fp, "n %d %d %d 0 360 arc \n",
	   circle->center_x, -circle->center_y, circle->radius);
  if (circle->fill)
    fprintf (fp, "fill\n");
  else
    fprintf (fp, "s\n");
}


void plot_string (DRAWOBJ *obj, FILE *fp)
{
  STRING *dstr = obj->object;
  SBOX *sbox = obj->sbox;
  int sx, sy, sw, sh;
  
  fprintf (fp, "TEXTFNT f (%s) %d %d t\n", dstr->str, dstr->x, -dstr->y);
}


