/* 
 * file-documentation
 * =========================================================================
 * types.h -- Type definitions of BACKDRAW
 * 
 * Author          : Hans Fleurkens
 * Created On      : Tue Jan 28 15:24:14 1992
 * Last Modified By: Hans Fleurkens
 * Last Modified On: Tue Sep 28 09:24:49 1993
 * Update Count    : 2
 * Status          : none
 * 
 * History
 * Description:
 * 	>
 * =========================================================================
 * Copyright (c) 1992-1993
 * All Rights Reserved
 * Hans Fleurkens
 * Eindhoven University of Technology
 * The Netherlands
 * =========================================================================
 */

typedef struct {
  Bool help;
  Bool verbose;
  Bool interactive;
  int nr_planes;
  int update;
  XFontStruct *drawfont;
  Bool plane_gc;
  String plot_dir;
} ApplicationData, *ApplicationDataPtr;


typedef struct PROMPT_DATA {
  Widget prompt_dialog;
  char *default_str;
  char *question_str;
  void *client_data;
  void (*ok_func) ();
  void (*cancel_func) ();
} PROMPT_DATA;

typedef struct PLANE {
  struct GCDATA *gcdata;
  struct DRAWOBJ *startp;
} PLANE;

typedef struct CBOX {
  int x1;
  int y1;
  int x2;
  int y2;
} CBOX;

typedef struct SBOX {
  int x;
  int y;
  int width;
  int height;
} SBOX;

typedef struct _CMDLIST {
  char *name;
  int (*function)();
} CMDLIST;

typedef struct GCDATA {
  GC gc;
  unsigned int red;
  unsigned int green;
  unsigned int blue;
  Pixel def_pixel;		/* default pixel value */
  Pixel gc_pixel;  		/* pixel set in gc     */
  XFontStruct *def_font; 	/* default font        */
  XFontStruct *gc_font;		/* font set in gc      */
} GCDATA;


typedef struct DRAWSTATUS {
  int plane;
  struct GCDATA *gcdata;
} DRAWSTATUS;


typedef struct DRAWOBJ {
  int type;
  struct SBOX *sbox;
  void *object;
  void (*drawfunc) ();
  void (*plotfunc) ();
  Pixel pixel;
  struct DRAWOBJ *prev;
  struct DRAWOBJ *next;
} DRAWOBJ;


typedef struct RECT {
  int fill;
} RECT;

typedef struct CIRCLE {
  int center_x;
  int center_y;
  int radius;
  int fill;
} CIRCLE;


typedef struct STRING {
  int x; /* real position */
  int y;
  XCharStruct *info;
  char *str;
  XFontStruct *font;
} STRING;


typedef struct LINE {
  int x1;
  int y1;
  int x2;
  int y2;
} LINE;

#ifdef COMMENT
typedef struct POINT {
  int x;
  int y;
} POINT;
#endif


typedef struct EHANDLER {
  /* Widget *canvas; necessary because canvas is still NULL at init */
  EventMask  event_mask;
  Boolean  nonmaskable;
  XtEventHandler  proc;
  XtPointer  client_data;
  XtListPosition position;
} EHANDLER;


